﻿using Coscine.Configuration;
using LinqToDB.Configuration;
using System.Collections.Generic;
using System.Linq;

namespace Coscine.Database.Settings
{
    public class CoscineSettings : ILinqToDBSettings
    {

        public CoscineSettings(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public string DbDataSourceKey { get; set; } = "coscine/global/db_data_source";
        public string DbNameKey { get; set; } = "coscine/global/db_name";
        public string DbUserIdKey { get; set; } = "coscine/global/db_user_id";
        public string DbPasswordKey { get; set; } = "coscine/global/db_password";

        public IConfiguration Configuration { get; set; }

        public IEnumerable<IDataProviderSettings> DataProviders => Enumerable.Empty<IDataProviderSettings>();

        public string DefaultConfiguration => "SqlServer";
        public string DefaultDataProvider => "SqlServer";

        public IEnumerable<IConnectionStringSettings> ConnectionStrings
        {
            get
            {
                var dbDataSource = Configuration.GetStringAndWait(DbDataSourceKey);
                var dbDatabase = Configuration.GetStringAndWait(DbNameKey);
                var dbUserId = Configuration.GetStringAndWait(DbUserIdKey);
                var dbPassword = Configuration.GetStringAndWait(DbPasswordKey);

                yield return
                    new ConnectionStringSettings
                    {
                        Name = "Coscine",
                        ProviderName = "SqlServer",
                        ConnectionString = $"Data Source={dbDataSource}; Database={dbDatabase}; User Id={dbUserId}; Password={dbPassword};"
                };
            }
        }
    }
}